import datetime
import re
import dateparser
import typing
from telegram import Bot as PTBBot
from telegram import Message as PTBMessage
from telegram.error import BadRequest, Unauthorized
from telegram import InlineKeyboardMarkup as InKeMa
from telegram import InlineKeyboardButton as InKeBu
from royalnet.commands import *
from royalnet.bots import TelegramBot
from royalnet.utils import telegram_escape, asyncify, sleep_until
from ..tables import MMEvent, MMResponse, User
from ..utils import MMChoice, MMInterfaceDataTelegram


class MatchmakingCommand(Command):
    name: str = "matchmaking"

    description: str = "Cerca persone per una partita a qualcosa!"

    syntax: str = "[ {ora} ] {nome}\n[descrizione]"

    aliases = ["mm", "lfg"]

    tables = {MMEvent, MMResponse}

    def __init__(self, interface: CommandInterface):
        super().__init__(interface)
        # Find all relevant MMEvents and run them
        session = self.alchemy.Session()
        mmevents = (
            session
                .query(self.alchemy.MMEvent)
                .filter(self.alchemy.MMEvent.interface == self.interface.name,
                        self.alchemy.MMEvent.datetime > datetime.datetime.now())
                .all()
        )
        for mmevent in mmevents:
            self.interface.loop.create_task(self._run_mmevent(mmevent.mmid))

    async def run(self, args: CommandArgs, data: CommandData) -> None:
        # Create a new MMEvent and run it
        if self.interface.name != "telegram":
            raise UnsupportedError(f"{self.interface.prefix}matchmaking funziona solo su Telegram. Per ora.")
        author = await data.get_author(error_if_none=True)

        try:
            timestring, title, description = args.match(r"\[\s*([^]]+)\s*]\s*([^\n]+)\s*\n?\s*(.+)?\s*", re.DOTALL)
        except InvalidInputError:
            timestring, title, description = args.match(r"\s*(.+?)\s*\n\s*([^\n]+)\s*\n?\s*(.+)?\s*", re.DOTALL)
        try:
            dt: typing.Optional[datetime.datetime] = dateparser.parse(timestring, settings={
                "PREFER_DATES_FROM": "future"
            })
        except OverflowError:
            dt = None
        if dt is None:
            raise CommandError("La data che hai specificato non è valida.")
        if dt <= datetime.datetime.now():
            raise CommandError("La data che hai specificato è nel passato.")
        if dt >= datetime.datetime.now() + datetime.timedelta(days=90):
            raise CommandError("La data che hai specificato è a più di 90 giorni di distanza da oggi.")
        mmevent: MMEvent = self.interface.alchemy.MMEvent(creator=author,
                                                          datetime=dt,
                                                          title=title,
                                                          description=description,
                                                          interface=self.interface.name)
        data.session.add(mmevent)
        await data.session_commit()
        self.interface.loop.create_task(self._run_mmevent(mmevent.mmid))
        await data.reply(f"✅ Evento [b]{mmevent.title}[/b] creato!")

    _mm_chat_id = -1001287169422

    _mm_error_chat_id = -1001153723135

    def _gen_mm_message(self, mmevent: MMEvent) -> str:
        text = f"🌐 [{mmevent.datetime.strftime('%Y-%m-%d %H:%M')}] [b]{mmevent.title}[/b]\n"
        if mmevent.description:
            text += f"{mmevent.description}\n"
        text += "\n"
        for response in mmevent.responses:
            response: MMResponse
            text += f"{response.choice.value} {response.user}\n"
        return text

    def _gen_telegram_keyboard(self, mmevent: MMEvent):
        return InKeMa([
            [InKeBu(f"{MMChoice.YES.value} Ci sarò!", callback_data=f"mm{mmevent.mmid}_YES")],
            [InKeBu(f"{MMChoice.MAYBE.value} (Forse.)", callback_data=f"mm{mmevent.mmid}_MAYBE")],
            [InKeBu(f"{MMChoice.LATE_SHORT.value} Arrivo dopo 5-10 min.",
                    callback_data=f"mm{mmevent.mmid}_LATE_SHORT")],
            [InKeBu(f"{MMChoice.LATE_MEDIUM.value} Arrivo dopo 15-35 min.",
                    callback_data=f"mm{mmevent.mmid}_LATE_MEDIUM")],
            [InKeBu(f"{MMChoice.LATE_LONG.value} Arrivo dopo 40+ min.", callback_data=f"mm{mmevent.mmid}_LATE_LONG")],
            [InKeBu(f"{MMChoice.NO_TIME.value} Non posso a quell'ora...", callback_data=f"mm{mmevent.mmid}_NO_TIME")],
            [InKeBu(f"{MMChoice.NO_INTEREST.value} Non mi interessa.", callback_data=f"mm{mmevent.mmid}_NO_INTEREST")],
            [InKeBu(f"{MMChoice.NO_TECH.value} Ho un problema!", callback_data=f"mm{mmevent.mmid}_NO_TECH")],
        ])

    async def _update_telegram_mm_message(self, client: PTBBot, mmevent: MMEvent):
        try:
            await self.interface.bot.safe_api_call(client.edit_message_text,
                                                   chat_id=self._mm_chat_id,
                                                   text=telegram_escape(self._gen_mm_message(mmevent)),
                                                   message_id=mmevent.interface_data.message_id,
                                                   parse_mode="HTML",
                                                   disable_web_page_preview=True,
                                                   reply_markup=self._gen_telegram_keyboard(mmevent))
        except BadRequest:
            pass

    def _gen_mm_telegram_callback(self, client: PTBBot, mmid: int, choice: MMChoice):
        async def callback(data: CommandData):
            author = await data.get_author(error_if_none=True)
            # Find the MMEvent with the current session
            mmevent: MMEvent = await asyncify(data.session.query(self.alchemy.MMEvent).get, mmid)
            mmresponse: MMResponse = await asyncify(
                data.session.query(self.alchemy.MMResponse).filter_by(user=author, mmevent=mmevent).one_or_none)
            if mmresponse is None:
                mmresponse = self.alchemy.MMResponse(user=author, mmevent=mmevent, choice=choice)
                data.session.add(mmresponse)
            else:
                mmresponse.choice = choice
            await data.session_commit()
            await self._update_telegram_mm_message(client, mmevent)
            return f"✅ Messaggio ricevuto!"

        return callback

    def _gen_event_start_message(self, mmevent: MMEvent):
        text = f"🚩 L'evento [b]{mmevent.title}[/b] è iniziato!\n\n"
        for response in mmevent.responses:
            response: MMResponse
            text += f"{response.choice.value} {response.user}\n"
        return text

    def _gen_unauth_message(self, user: User):
        return f"⚠️ Non sono autorizzato a mandare messaggi a [b]{user.username}[/b]!\n" \
               f"{user.telegram.mention()}, apri una chat privata con me e mandami un messaggio!"

    async def _run_mmevent(self, mmid: int):
        """Run a MMEvent."""
        # Open a new Alchemy Session
        session = self.alchemy.Session()
        # Find the MMEvent with the current session
        mmevent: MMEvent = await asyncify(session.query(self.alchemy.MMEvent).get, mmid)
        if mmevent is None:
            raise ValueError("Invalid mmid.")
        # Ensure the MMEvent hasn't already started
        if mmevent.datetime <= datetime.datetime.now():
            raise ValueError("MMEvent has already started.")
        # Ensure the MMEvent interface matches the current one
        if mmevent.interface != self.interface.name:
            raise ValueError("Invalid interface.")
        # If the matchmaking message hasn't been sent yet, do so now
        if mmevent.interface_data is None:
            if self.interface.name == "telegram":
                bot: TelegramBot = self.interface.bot
                client: PTBBot = bot.client
                # Build the Telegram keyboard
                # Send the keyboard
                message: PTBMessage = await self.interface.bot.safe_api_call(client.send_message,
                                                                             chat_id=self._mm_chat_id,
                                                                             text=telegram_escape(
                                                                                 self._gen_mm_message(mmevent)),
                                                                             parse_mode="HTML",
                                                                             disable_webpage_preview=True,
                                                                             reply_markup=self._gen_telegram_keyboard(
                                                                                 mmevent))
                # Store message data in the interface data object
                mmevent.interface_data = MMInterfaceDataTelegram(chat_id=self._mm_chat_id,
                                                                 message_id=message.message_id)
                await asyncify(session.commit)
            else:
                raise UnsupportedError()
        # Register handlers for the keyboard events
        if self.interface.name == "telegram":
            bot: TelegramBot = self.interface.bot
            client: PTBBot = bot.client
            self.interface.register_keyboard_key(f"mm{mmevent.mmid}_YES",
                                                 callback=self._gen_mm_telegram_callback(client, mmid, MMChoice.YES))
            self.interface.register_keyboard_key(f"mm{mmevent.mmid}_MAYBE",
                                                 callback=self._gen_mm_telegram_callback(client, mmid, MMChoice.MAYBE))
            self.interface.register_keyboard_key(f"mm{mmevent.mmid}_LATE_SHORT",
                                                 callback=self._gen_mm_telegram_callback(client, mmid,
                                                                                         MMChoice.LATE_SHORT))
            self.interface.register_keyboard_key(f"mm{mmevent.mmid}_LATE_MEDIUM",
                                                 callback=self._gen_mm_telegram_callback(client, mmid,
                                                                                         MMChoice.LATE_MEDIUM))
            self.interface.register_keyboard_key(f"mm{mmevent.mmid}_LATE_LONG",
                                                 callback=self._gen_mm_telegram_callback(client, mmid,
                                                                                         MMChoice.LATE_LONG))
            self.interface.register_keyboard_key(f"mm{mmevent.mmid}_NO_TIME",
                                                 callback=self._gen_mm_telegram_callback(client, mmid,
                                                                                         MMChoice.NO_TIME))
            self.interface.register_keyboard_key(f"mm{mmevent.mmid}_NO_INTEREST",
                                                 callback=self._gen_mm_telegram_callback(client, mmid,
                                                                                         MMChoice.NO_INTEREST))
            self.interface.register_keyboard_key(f"mm{mmevent.mmid}_NO_TECH",
                                                 callback=self._gen_mm_telegram_callback(client, mmid,
                                                                                         MMChoice.NO_TECH))
        else:
            raise UnsupportedError()
        # Sleep until the time of the event
        await sleep_until(mmevent.datetime)
        # Notify the positive answers of the event start
        if self.interface.name == "telegram":
            bot: TelegramBot = self.interface.bot
            client: PTBBot = bot.client
            self.interface.unregister_keyboard_key(f"mm{mmevent.mmid}_YES")
            self.interface.unregister_keyboard_key(f"mm{mmevent.mmid}_MAYBE")
            self.interface.unregister_keyboard_key(f"mm{mmevent.mmid}_LATE_SHORT")
            self.interface.unregister_keyboard_key(f"mm{mmevent.mmid}_LATE_MEDIUM")
            self.interface.unregister_keyboard_key(f"mm{mmevent.mmid}_LATE_LONG")
            self.interface.unregister_keyboard_key(f"mm{mmevent.mmid}_NO_TIME")
            self.interface.unregister_keyboard_key(f"mm{mmevent.mmid}_NO_INTEREST")
            self.interface.unregister_keyboard_key(f"mm{mmevent.mmid}_NO_TECH")
            for response in mmevent.responses:
                if response.choice == MMChoice.NO_INTEREST or response.choice == MMChoice.NO_TIME:
                    return
                try:
                    await self.interface.bot.safe_api_call(client.send_message,
                                                           chat_id=response.user.telegram[0].tg_id,
                                                           text=telegram_escape(self._gen_event_start_message(mmevent)),
                                                           parse_mode="HTML",
                                                           disable_webpage_preview=True)
                except Unauthorized:
                    await self.interface.bot.safe_api_call(client.send_message,
                                                           chat_id=self._mm_error_chat_id,
                                                           text=telegram_escape(
                                                               self._gen_unauth_message(response.user)),
                                                           parse_mode="HTML",
                                                           disable_webpage_preview=True)
        else:
            raise UnsupportedError()
        # Delete the event message
        if self.interface.name == "telegram":
            await self.interface.bot.safe_api_call(client.delete_message,
                                                   chat_id=mmevent.interface_data.chat_id,
                                                   message_id=mmevent.interface_data.message_id)
        # The end!
        await asyncify(session.close)



import typing
import dateparser
import datetime
import pickle
import telegram
import discord
from sqlalchemy import and_
from royalnet.commands import *
from royalnet.utils import *
from royalnet.serf.telegram import escape as telegram_escape
from royalnet.serf.discord import escape as discord_escape
from ..tables import Reminder


class ReminderCommand(Command):
    name: str = "reminder"

    aliases = ["calendar"]

    description: str = "Ti ricorda di fare qualcosa dopo un po' di tempo."

    syntax: str = "[ {data} ] {messaggio}"

    def __init__(self, interface: CommandInterface):
        super().__init__(interface)
        session = interface.alchemy.Session()
        reminders = (
            session.query(interface.alchemy.get(Reminder))
                   .filter(and_(
                       interface.alchemy.get(Reminder).datetime >= datetime.datetime.now(),
                       interface.alchemy.get(Reminder).interface_name == interface.name))
                   .all()
        )
        for reminder in reminders:
            interface.loop.create_task(self._remind(reminder))

    async def _remind(self, reminder):
        await sleep_until(reminder.datetime)
        if self.interface.name == "telegram":
            chat_id: int = pickle.loads(reminder.raw_interface_data)
            client: telegram.Bot = self.serf.client
            await self.serf.api_call(client.send_message,
                                     chat_id=chat_id,
                                     text=telegram_escape(f"❗️ {reminder.message}"),
                                     parse_mode="HTML",
                                     disable_web_page_preview=True)
        elif self.interface.name == "discord":
            channel_id: int = pickle.loads(reminder.raw_interface_data)
            client: discord.Client = self.serf.client
            channel = client.get_channel(channel_id)
            await channel.send(discord_escape(f"❗️ {reminder.message}"))

    async def run(self, args: CommandArgs, data: CommandData) -> None:
        try:
            date_str, reminder_text = args.match(r"\[\s*([^]]+)\s*]\s*([^\n]+)\s*")
        except InvalidInputError:
            date_str, reminder_text = args.match(r"\s*(.+?)\s*\n\s*([^\n]+)\s*")

        try:
            date: typing.Optional[datetime.datetime] = dateparser.parse(date_str, settings={
                "PREFER_DATES_FROM": "future"
            })
        except OverflowError:
            date = None
        if date is None:
            await data.reply("⚠️ La data che hai inserito non è valida.")
            return
        if date <= datetime.datetime.now():
            await data.reply("⚠️ La data che hai specificato è nel passato.")
            return
        await data.reply(f"✅ Promemoria impostato per [b]{date.strftime('%Y-%m-%d %H:%M:%S')}[/b]")
        if self.interface.name == "telegram":
            interface_data = pickle.dumps(data.update.effective_chat.id)
        elif self.interface.name == "discord":
            interface_data = pickle.dumps(data.message.channel.id)
        else:
            raise UnsupportedError("This command does not support the current interface.")
        creator = await data.get_author()
        reminder = self.interface.alchemy.Reminder(creator=creator,
                                                   interface_name=self.interface.name,
                                                   interface_data=interface_data,
                                                   datetime=date,
                                                   message=reminder_text)
        self.interface.loop.create_task(self._remind(reminder))
        data.session.add(reminder)
        await asyncify(data.session.commit)



from starlette.requests import Request
from starlette.responses import *
from royalnet.constellation import *
from royalnet.utils import *
from ..tables import *


class ApiDiarioListStar(PageStar):
    path = "/api/diario/list"

    async def page(self, request: Request) -> JSONResponse:
        page_str = request.query_params.get("page", "0")
        try:
            page = int(page_str)
        except (ValueError, TypeError):
            return shoot(400, "Invalid offset")
        async with self.alchemy.session_acm() as session:
            if page < 0:
                page = -page-1
                entries: typing.List[Diario] = await asyncify(
                    session.query(self.alchemy.get(Diario))
                           .order_by(self.alchemy.get(Diario).diario_id.desc()).limit(500)
                           .offset(page * 500)
                           .all
                )
            else:
                entries: typing.List[Diario] = await asyncify(
                    session.query(self.alchemy.get(Diario))
                           .order_by(self.alchemy.get(Diario).diario_id)
                           .limit(500)
                           .offset(page * 500)
                           .all)
            response = [entry.json() for entry in entries]
        return JSONResponse(response)



from sqlalchemy import Column, \
                       Integer, \
                       Text, \
                       ForeignKey
from sqlalchemy.orm import relationship, backref
from sqlalchemy.ext.declarative import declared_attr


class Bio:
    __tablename__ = "bios"

    @declared_attr
    def royal_id(self):
        return Column(Integer, ForeignKey("users.uid"), primary_key=True)

    @declared_attr
    def royal(self):
        return relationship("User", backref=backref("bio", uselist=False))

    @declared_attr
    def contents(self):
        return Column(Text, nullable=False, default="")

    def __repr__(self):
        return f"<Bio of {self.royal}>"

    def __str__(self):
        return self.contents



# Imports go here!
from .api_user_list import ApiUserListStar
from .api_user_get import ApiUserGetStar
from .api_diario_list import ApiDiarioListStar
from .api_diario_get import ApiDiarioGetStar
# from .api_discord_cv import ApiDiscordCvStar

# Enter the PageStars of your Pack here!
available_page_stars = [
    ApiUserListStar,
    ApiUserGetStar,
    ApiDiarioListStar,
    ApiDiarioGetStar,
    # ApiDiscordCvStar,
]

# Enter the ExceptionStars of your Pack here!
available_exception_stars = [

]

# Don't change this, it should automatically generate __all__
__all__ = [star.__name__ for star in [*available_page_stars, *available_exception_stars]]



from .play import PlayCommand


class YoutubeCommand(PlayCommand):
    name: str = "youtube"

    aliases = ["yt"]

    description: str = "Cerca un video su YouTube e lo aggiunge alla coda della chat vocale."

    syntax = "{ricerca}"

    _URL_FORMAT = "ytsearch:{url}"



from sqlalchemy import *
from sqlalchemy.orm import relationship
from sqlalchemy.ext.declarative import declared_attr
from ..utils import MMChoice


class MMResponse:
    __tablename__ = "mmresponse"

    @declared_attr
    def user_id(self):
        return Column(Integer, ForeignKey("users.uid"), primary_key=True)

    @declared_attr
    def user(self):
        return relationship("User", backref="mmresponses_given")

    @declared_attr
    def mmevent_id(self):
        return Column(Integer, ForeignKey("mmevents.mmid"), primary_key=True)

    @declared_attr
    def mmevent(self):
        return relationship("MMEvent", backref="responses")

    @declared_attr
    def choice(self):
        return Column(Enum(MMChoice), nullable=False)

    def __repr__(self):
        return f"<MMResponse of {self.user}: {self.choice}>"



from starlette.requests import Request
from starlette.responses import *
from royalnet.constellation import *
from royalnet.utils import *


class ApiDiscordCvStar(PageStar):
    path = "/api/discord/cv"

    async def page(self, request: Request) -> JSONResponse:
        response = await self.constellation.call_herald_event("discord", "discord_cv")
        return JSONResponse(response)



# This is a template Pack __init__. You can use this without changing anything in other packages too!

from . import commands, tables, stars, events
from .commands import available_commands
from .tables import available_tables
from .stars import available_page_stars, available_exception_stars
from .events import available_events

from .version import semantic as __version__

__all__ = [
    "commands",
    "tables",
    "stars",
    "events",
    "available_commands",
    "available_tables",
    "available_page_stars",
    "available_exception_stars",
    "available_events",
]



from .play import PlayCommand


class YahoovideoCommand(PlayCommand):
    name: str = "yahoovideo"

    aliases = ["yv"]

    description: str = "Cerca un video su Yahoo Video e lo aggiunge alla coda della chat vocale."

    syntax = "{ricerca}"

    _URL_FORMAT = "yvsearch:{url}"

    # Too bad yvsearch: always finds nothing.



import discord
import pickle
import base64
import datetime
from typing import *
from royalnet.commands import *
from royalnet.serf.discord import *
from royalnet.bard import *
from ..utils import RoyalQueue


class DiscordPlayEvent(Event):
    name = "discord_play"

    async def run(self,
                  url: str,
                  guild_id: Optional[int] = None,
                  **kwargs) -> dict:
        if not isinstance(self.serf, DiscordSerf):
            raise UnsupportedError()
        client: discord.Client = self.serf.client
        if len(self.serf.voice_players) == 1:
            voice_player: VoicePlayer = self.serf.voice_players[0]
        else:
            if guild_id is None:
                # TODO: trovare un modo per riprodurre canzoni su più server da Telegram
                raise InvalidInputError("Non so in che Server riprodurre questo file...\n"
                                        "Invia il comando su Discord, per favore!")
            guild: discord.Guild = client.get_guild(guild_id)
            if guild is None:
                raise InvalidInputError("Impossibile trovare il Server specificato.")
            voice_player: VoicePlayer = self.serf.find_voice_player(guild)
            if voice_player is None:
                raise UserError("Il bot non è in nessun canale vocale.\n"
                                "Evocalo prima con [c]summon[/c]!")
        ytds = await YtdlDiscord.from_url(url)
        added: List[YtdlDiscord] = []
        too_long: List[YtdlDiscord] = []
        if isinstance(voice_player.playing, RoyalQueue):
            for index, ytd in enumerate(ytds):
                if ytd.info.duration >= datetime.timedelta(seconds=self.config["Play"]["max_song_duration"]):
                    too_long.append(ytd)
                    continue
                await ytd.convert_to_pcm()
                added.append(ytd)
                voice_player.playing.contents.append(ytd)
            if not voice_player.voice_client.is_playing():
                await voice_player.start()
        else:
            raise CommandError(f"Non so come aggiungere musica a [c]{voice_player.playing.__class__.__qualname__}[/c]!")
        return {
            "added": [{
                "title": ytd.info.title,
                "stringified_base64_pickled_discord_embed": str(base64.b64encode(pickle.dumps(ytd.embed())),
                                                                encoding="ascii")
            } for ytd in added],
            "too_long": [{
                "title": ytd.info.title,
                "stringified_base64_pickled_discord_embed": str(base64.b64encode(pickle.dumps(ytd.embed())),
                                                                encoding="ascii")
            } for ytd in too_long]
        }



# Imports go here!
from .diario import Diario
from .aliases import Alias
from .wikipages import WikiPage
from .wikirevisions import WikiRevision
from .bios import Bio
from .reminders import Reminder
from .triviascores import TriviaScore
from .mmevents import MMEvent
from .mmresponse import MMResponse
from .leagueoflegends import LeagueOfLegends

# Enter the tables of your Pack here!
available_tables = [
    Diario,
    Alias,
    WikiPage,
    WikiRevision,
    Bio,
    Reminder,
    TriviaScore,
    MMEvent,
    MMResponse,
    LeagueOfLegends
]

# Don't change this, it should automatically generate __all__
__all__ = [table.__name__ for table in available_tables]



from typing import *
from royalnet.commands import *
from royalnet.utils import *
from royalnet.backpack.tables import User
from sqlalchemy import func
import royalspells as rs


class SpellCommand(Command):
    name: str = "spell"

    description: str = "Genera casualmente una spell!"

    syntax = "{nome_spell}"

    async def run(self, args: CommandArgs, data: CommandData) -> None:
        spell_name = args.joined(require_at_least=1)
        spell = rs.Spell(spell_name)

        rows: List[str] = [f"✨ [b]{spell.name}[/b]"]

        if spell.damage_component:
            dmg: rs.DamageComponent = spell.damage_component
            constant_str: str = f"{dmg.constant:+d}" if dmg.constant != 0 else ""
            rows.append(f"Danni: [b]{dmg.dice_number}d{dmg.dice_type}{constant_str}[/b]"
                        f" {andformat(dmg.damage_types, final=' e ')}")
            rows.append(f"Precisione: [b]{dmg.miss_chance}%[/b]")
            if dmg.repeat > 1:
                rows.append(f"Multiattacco: [b]×{dmg.repeat}[/b]")
            rows.append("")

        if spell.healing_component:
            heal: rs.HealingComponent = spell.healing_component
            constant_str: str = f"{heal.constant:+d}" if heal.constant != 0 else ""
            rows.append(f"Cura: [b]{heal.dice_number}d{heal.dice_type}{constant_str}[/b] HP")
            rows.append("")

        if spell.stats_component:
            stats: rs.StatsComponent = spell.stats_component
            rows.append("Il caster riceve: ")
            for stat_name in stats.stat_changes:
                rows.append(f"[b]{stats.stat_changes[stat_name]}{stat_name}[/b]")
            rows.append("")

        if spell.status_effect_component:
            se: rs.StatusEffectComponent = spell.status_effect_component
            rows.append("Infligge al bersaglio: ")
            rows.append(f"[b]{se.effect}[/b] ({se.chance}%)")
            rows.append("")

        await data.reply("\n".join(rows))



from starlette.requests import Request
from starlette.responses import *
from royalnet.constellation import *
from royalnet.utils import *
from royalnet.backpack.tables import *


class ApiUserListStar(PageStar):
    path = "/api/user/list"
    tables = {User}

    async def page(self, request: Request) -> JSONResponse:
        async with self.alchemy.session_acm() as session:
            users: typing.List[User] = await asyncify(session.query(self.alchemy.get(User)).all)
        return JSONResponse([user.json() for user in users])



import pickle
import base64
import discord
from typing import *
from royalnet.commands import *
from royalnet.utils import *


class PlayCommand(Command):
    name: str = "play"

    aliases = ["p"]

    description: str = "Aggiunge un url alla coda della chat vocale."

    syntax = "{url}"

    _URL_FORMAT = "{url}"

    async def run(self, args: CommandArgs, data: CommandData) -> None:
        url = args.joined()
        # if not (url.startswith("http://") or url.startswith("https://")):
        #     raise CommandError(f"Il comando [c]{self.interface.prefix}play[/c] funziona solo per riprodurre file da"
        #                        f" un URL.\n"
        #                        f"Se vuoi cercare un video, come misura temporanea puoi usare "
        #                        f"[c]ytsearch:nomevideo[/c] o [c]scsearch:nomevideo[/c] come url.")
        if self.interface.name == "discord":
            message: discord.Message = data.message
            guild: discord.Guild = message.guild
            guild_id: Optional[int] = guild.id
        else:
            guild_id = None
        response: Dict[str, Any] = await self.interface.call_herald_event("discord", "discord_play",
                                                                          url=self._URL_FORMAT.format(url=url),
                                                                          guild_id=guild_id)

        too_long: List[Dict[str, Any]] = response["too_long"]
        if len(too_long) > 0:
            await data.reply(f"⚠ {len(too_long)} file non {'è' if len(too_long) == 1 else 'sono'}"
                             f" stat{'o' if len(too_long) == 1 else 'i'} scaricat{'o' if len(too_long) == 1 else 'i'}"
                             f" perchè durava{'' if len(too_long) == 1 else 'no'}"
                             f" più di [c]{self.config['Play']['max_song_duration']}[/c] secondi.")

        added: List[Dict[str, Any]] = response["added"]
        if len(added) > 0:
            reply = f"▶️ Aggiunt{'o' if len(added) == 1 else 'i'} {len(added)} file alla coda:\n"
            if self.interface.name == "discord":
                await data.reply(reply)
                for item in added:
                    embed = pickle.loads(base64.b64decode(bytes(item["stringified_base64_pickled_discord_embed"],
                                                                encoding="ascii")))
                    # noinspection PyUnboundLocalVariable
                    await message.channel.send(embed=embed)
            else:
                reply += numberemojiformat([a["title"] for a in added])
                await data.reply(reply)

        if len(added) + len(too_long) == 0:
            raise ExternalError("Nessun video trovato.")


from .mmchoice import MMChoice
from .mminterfacedata import MMInterfaceData, MMInterfaceDataTelegram
from .leaguetier import LeagueTier
from .leaguerank import LeagueRank
from .leagueleague import LeagueLeague
from .royalqueue import RoyalQueue

__all__ = [
    "MMChoice",
    "MMInterfaceData",
    "MMInterfaceDataTelegram",
    "LeagueTier",
    "LeagueRank",
    "LeagueLeague",
    "RoyalQueue",
]



import typing
import random
from royalnet.commands import *


class RageCommand(Command):
    name: str = "rage"

    aliases = ["balurage", "madden"]

    description: str = "Arrabbiati per qualcosa, come una software house californiana."

    _MAD = ["MADDEN MADDEN MADDEN MADDEN",
           "EA bad, praise Geraldo!",
           "Stai sfogando la tua ira sul bot!",
           "Basta, io cambio gilda!",
           "Fondiamo la RRYG!"]

    async def run(self, args: CommandArgs, data: CommandData) -> None:
        await data.reply(f"😠 {random.sample(self._MAD, 1)[0]}")



from royalnet.commands import *


class EatCommand(Command):
    name: str = "eat"

    description: str = "Mangia qualcosa!"

    syntax: str = "{cibo}"

    _FOODS = {
        "_default": "🍗 Hai mangiato {food}!\n[i]Ma non è successo nulla.[/i]",
        
        # Sezione nonna
        "tutto": "👵🏻 Hai mangiato {food}. Si vede che hai gradito il pasto!\n[i]Tua nonna ti serve un'altra"
                 " porzione.[/i]",
        "poco": "👵🏻 Hai mangiato davvero {food}! \n[i]Tua nonna è molto arrabbiata e ferita nell'orgoglio."
                " Vergognati![/i]",
        "nonna": "👵🏻 Hai mangiato tua {food}. In qualche modo, continua a cucinarti cibo anche da dentro la"
                 " pancia.\n[i]This can't end well...[/i]",
        "qualcosa di non cucinato dalla nonna": "👵🏻 Hai mangiato {food}!\n[i]Potresti essere appena stato "
                                                "diseredato...[/i]",
        "qualcosa di non preparato dalla nonna": "👵🏻 Hai mangiato {food}!\n[i]Potresti essere appena stato "
                                                "diseredato...[/i]",
        
        # Sezione caffé (ce n'erano troppi, con gli accenti e il resto)
        "caffè": "☕️ Oh, no! Questo era il {food} della Peppina!\n[i]Ha provato col tritolo, salti in aria col"
                 " {food}.[/i]",
        "caffé": "☕️ Oh, no! Questo era il {food} della Peppina!\n[i]Ha provato col tritolo, salti in aria col"
                 " {food}.[/i]",
        "caffe": "☕️ Oh, no! Questo era il {food} della Peppina!\n[i]Ha provato col tritolo, salti in aria col"
                 " {food}.[/i]",
        "kaffee": "☕️ Ma BUONGIORNISSIMOOO !!!!\n[i]Non si può iniziare la giornata senza un buon {food} !![/i]",
        "kaffè": "☕️ Ma BUONGIORNISSIMOOO !!!!\n[i]Non si può iniziare la giornata senza un buon {food} !![/i]",
        "kaffé": "☕️ Ma BUONGIORNISSIMOOO !!!!\n[i]Non si può iniziare la giornata senza un buon {food} !![/i]",
        "kaffe": "☕️ Ma BUONGIORNISSIMOOO !!!!\n[i]Non si può iniziare la giornata senza un buon {food} !![/i]",
        
        # Sezione in cui mangi i membri Royal Games
        "balu": "🚹 Hai mangiato {food}.\n[i]Sa di snado.[/i]",
        "evilbalu": "🚹 Hai mangiato {food}.\n[i]Sa di snado.[/i]",
        "balubis": "🚹 Hai mangiato {food}.\n[i]Sa di acqua calda.[/i]",
        "goodbalu": "🚹 Hai mangiato {food}.\n[i]Sa di acqua calda.[/i]",
        "chiara": "🚺 Hai mangiato {food}.\n[i]Sa un po' di biscotto, ma per lo più sa di curcuma, pepe e spezie"
                  " varie.[/i]",
        "fabio": "🚹 Hai mangiato {food}.\n[i]Sa di gelatina tuttigusti+1.[/i]",
        "proto": "🚹 Hai mangiato {food}.\n[i]Sa di gelatina tuttigusti+1.[/i]",
        "marco": "🚹 Hai mangiato {food}.\n[i]Sa di carlino <.<[/i]",
        "mallllco": "🚹 Hai mangiato {food}.\n[i]Sa di carlino <.<[/i]",
        "max": "🚹 Hai mangiato {food}.\n[i]Sa di merda.[/i]",
        "maxsensei": "🚹 Hai mangiato {food}.\n[i]Sa di merda.[/i]",
        "steffo": "🚹 Hai mangiato {food}.\n[i]Sa di gelato e di Coca-Cola.[/i]",
        
        #Sezione delle supercazzole
        "antani": "Hai mangiato {food}. \n[i]Con tarapia tapioco o scherziamo? No, mi permetta. Noi siamo in 4."
        "Come se fosse antani anche per lei soltanto in due, oppure in quattro anche scribàcchi confaldina?"
        "Come antifurto, per esempio.[/i]",
        "indice": "Hai mangiato l'{food}. \n[i]Ecco, lo alzi. Lo vede, lo vede che stuzzica?[/i]",
        
        # Altro
        "tonnuooooooro": "👻 Il {food} che hai mangiato era posseduto.\n[i]Spooky![/i]",
        "uranio": "☢️ L'{food} che hai mangiato era radioattivo.\n[i]Stai brillando di verde![/i]",
        "pollo": '🍗 Il {food} che hai appena mangiato proveniva dallo spazio.\n[i]Coccodè?[/i]',
        "ragno": "🕸 Hai mangiato un {food}.\n[i]Ewww![/i]",
        "curry": "🔥 BRUCIAAAAAAAAAA! Il {food} era piccantissimo!\n[i]Stai sputando fiamme![/i]",
        "peperoncino": "🔥 BRUCIAAAAAAAAAA! Il {food} era piccantissimo!\n[i]Stai sputando fiamme![/i]",
        "torta": "⬜️ Non hai mangiato niente.\n[i]La {food} è una menzogna![/i]",
        "cake": "⬜️ Non hai mangiato niente.\n[i]The {food} is a lie![/i]",
        "biscotto": "🍪 Hai mangiato un {food} di contrabbando.\n[i]L'Inquisizione non lo saprà mai![/i]",
        "biscotti": "🍪 Hai mangiato tanti {food} di contrabbando.\n[i]Attento! L'Inquisizione è sulle tue tracce![/i]",
        "tango": "🌳 Hai mangiato un {food}, e un albero insieme ad esso.\n[i]Senti le tue ferite curarsi...[/i]",
        "giaroun": "🥌 Il {food} che hai mangiato era duro come un {food}.\n[i]Stai soffrendo di indigestione![/i]",
        "giarone": "🥌 Il {food} che hai mangiato era duro come un {food}.\n[i]Stai soffrendo di indigestione![/i]",
        "sasso": "🥌 Il {food} che hai mangiato era duro come un {food}.\n[i]Stai soffrendo di indigestione![/i]",
        "gnocchetti": "🥘 Ullà, sono duri 'sti {food}!\n[i]Fai fatica a digerirli.[/i]",
        "tide pod": "☣️ I {food} che hai mangiato erano buonissimi.\n[i]Stai sbiancando![/i]",
        "tide pods": "☣️ I {food} che hai mangiato erano buonissimi.\n[i]Stai sbiancando![/i]",
        "gelato": "🍨 Mangiando del {food}, hai invocato Steffo.\n[i]Cedigli ora il tuo gelato.[/i]",
        "tua mamma": "⚠️ Non sei riuscito a mangiare {food}.\n[i]Era troppo grande e non ci stava nella tua bocca![/i]",
        "mango": "🥭 Hai mangiato un {food}.\n[i]Ti sembra di avere più mana, adesso.[/i]",
        "mango incantato": "🥭 Hai mangiato un {food}.\n[i]Ti sembra di avere più mana, adesso.[/i]",
        "enchanted mango": "🥭 Hai mangiato un {food}.\n[i]Ti sembra di avere più mana, adesso.[/i]",
        "musica": "🎶 Hai mangiato un po' di {food} mentre ascoltavi un buon pranzo.\n[i]Tutto ciò ha perfettamente"
                  " senso.[/i]",
        "fungo": "🍄 Hai mangiato un {food}.\n[i]Presto riuscirai a salvare Peach![/i]",
        "zucca": "🎃 Hai mangiato una {food}. Solo che era una lanterna di Halloween.\n[i]Inizi a fare luce al"
                 " buio.[/i]",
        "granchio": "🦀 Hai mangiato un {food}. {food} is gone!\n[i]Senti il tuo stomaco ballare.[/i]",
        "crab": "🦀 Hai mangiato un {food}. {food} is gone!\n[i]Senti il tuo stomaco ballare.[/i]",
        "veleno": "☠️ Hai mangiato del {food}. Perché lo hai fatto?\n[i]Adesso stai male, contento?[/i]",
        "polvere": "☁️ Hai mangiato la {food}.\n[i]Ti hanno proprio battuto![/i]",
        "diavolo": "👿 Hai mangiato un {food}. Non l'ha presa bene...\n[i]Hai terribili bruciori di stomaco.[/i]",
        "demone": "👿 Hai mangiato un {food}. Non l'ha presa bene...\n[i]Hai terribili bruciori di stomaco.[/i]",
        "niente": "⬜️ Non hai mangiato {food}.\n[i]Hai ancora più fame.[/i]",
        "nulla": "⬜️ Non hai mangiato {food}.\n[i]Hai ancora più fame.[/i]",
        "tre porcellini": "🐷 Hai mangiato i {food}.\n[i]La casa di mattoni non è bastata a fermarti![/i]",
        "3 porcellini": "🐷 Hai mangiato i {food}.\n[i]La casa di mattoni non è bastata a fermarti![/i]",
        "gatto": "🐱 Vieni fermato prima di poter compiere questo gesto orribile.\n"
                 "[i]Il {food} verrà pettato da tutti per farlo riavere dal trauma.[/i]",
        "riso": "🍚 Hai mangiato del {food}. Non ci resta che il Pianto! \n[i]Ba dum tsss![/i]",
        "royal bot": "🤖 Come osi provare a mangiarmi?!\n[i]Il {food} è arrabbiato con te.[/i]",
        "bot": "🤖 Come osi provare a mangiarmi?!\n[i]Il {food} è arrabbiato con te.[/i]",
        "mela": "🍎 Hai mangiato una Mela, e hai fatto bene perché una mela al giorno toglie il medico di torno!\n"
                "[i]Adesso sei molto più sano.[/i]",
        "redbull": "🍾 Hai mangiato {food}.\n[i]Adesso puoi volare![/i]",
        "red bull": "🍾 Hai mangiato {food}.\n[i]Adesso puoi volare![/i]",
        "birra": "🍺 Hai mangiato {food}.\n[i]Adesso sei un povero barbone alcolizzato.[/i]",
        "cacca": "💩 Che schifo! Hai mangiato {food}!\n[i]Allontati per favore, PLEH![/i]",
        "cochina": "🥫 Hai bevuto una {food}. \n[i]Bella fresca.[/i]",
        "little salami": "🥓 Mmmh, tasty!\n[i]Cats can have {food} too![/i]",
        "a little salami": "🥓 Mmmh, tasty!\n[i]Cats can have {food} too![/i]",
        "acqua": "💧 Hai bevuto un po' d'{food}.\n[i]Ti depura e ti fa fare tanta plin plin![/i}",
        "insetto": "🐞 Hai mangiato un {food}.\n[i]Dicono che sia il cibo del futuro, però fa un po' schifo.[/i]",
        "bug": "👾 Bravo, hai mangiato un {food}! Il tuo programma funziona un po' meglio.\n[i]Il problema è che"
               "ne sono comparsi altri tre.[/i]",
        "troppo": "⚠️ Hai mangiato {food}!\n[i]Hai un terribile mal di pancia.[/i]",
        "lemon": "🍋 Life gave you {food}s, so you ate them!\n[i]Sono un po' bruschi, ma commestibili.[/i]",
        "lemons": "🍋 Life gave you {food}, so you ate them!\n[i]Sono un po' bruschi, ma commestibili.[/i]",
        "bomba": "💣 Hai mangiato una {food}. Speriamo fosse solo calorica!\n[i]3... 2... 1...[/i]",
        "re": "👑 Hai mangiato il {food} avversario! \n[i]Scacco matto![/i]",
        "cactus": "🌵 Hai mangiato un {food}.\n[i]Gli hai tolto le spine prima, vero?[/i]",
        "kiwi": "🥝 Hai mangiato un {food}!\n[i]Li uoi qvei k-\n Li vuoi kuei uiw- \n Vabbè, avete capito![/i]",
    }

    async def run(self, args: CommandArgs, data: CommandData) -> None:
        food = args.joined(require_at_least=1)
        food_string = self._FOODS.get(food.lower(), self._FOODS["_default"])
        await data.reply(food_string.format(food=food.capitalize()))



import typing
import discord
from royalnet.commands import *


class VideochannelCommand(Command):
    name: str = "videochannel"

    aliases = ["golive", "live", "video"]

    description: str = "Converti il canale vocale in un canale video."

    syntax = "[nomecanale]"

    async def run(self, args: CommandArgs, data: CommandData) -> None:
        if self.interface.name != "discord":
            raise UnsupportedError(f"{self} non è supportato su {self.interface.name}.")
        bot: discord.Client = self.serf.client
        message: discord.Message = data.message
        channel_name: str = args.optional(0)
        if channel_name:
            guild: typing.Optional[discord.Guild] = message.guild
            if guild is not None:
                channels: typing.List[discord.abc.GuildChannel] = guild.channels
            else:
                channels = bot.get_all_channels()
            matching_channels: typing.List[discord.VoiceChannel] = []
            for channel in channels:
                if isinstance(channel, discord.VoiceChannel):
                    if channel.name == channel_name:
                        matching_channels.append(channel)
            if len(matching_channels) == 0:
                raise InvalidInputError("Non esiste alcun canale vocale con il nome specificato.")
            elif len(matching_channels) > 1:
                raise UserError("Esiste più di un canale vocale con il nome specificato.")
            channel = matching_channels[0]
        else:
            author: discord.Member = message.author
            voice: typing.Optional[discord.VoiceState] = author.voice
            if voice is None:
                raise InvalidInputError("Non sei connesso a nessun canale vocale.")
            channel = voice.channel
            if author.is_on_mobile():
                await data.reply(f"📹 Per entrare in modalità video, clicca qui:\n"
                                 f"<https://discordapp.com/channels/{channel.guild.id}/{channel.id}>\n"
                                 f"[b]Attenzione: la modalità video non funziona su Android e iOS![/b]")
                return
        await data.reply(f"📹 Per entrare in modalità video, clicca qui:\n"
                         f"<https://discordapp.com/channels/{channel.guild.id}/{channel.id}>")



from starlette.requests import Request
from starlette.responses import *
from royalnet.constellation import *
from royalnet.utils import *
from ..tables import *


class ApiDiarioGetStar(PageStar):
    path = "/api/diario/get/{diario_id}"

    async def page(self, request: Request) -> JSONResponse:
        diario_id_str = request.path_params.get("diario_id", "")
        try:
            diario_id = int(diario_id_str)
        except (ValueError, TypeError):
            return shoot(400, "Invalid diario_id")
        async with self.alchemy.session_acm() as session:
            entry: Diario = await asyncify(session.query(self.alchemy.get(Diario)).get, diario_id)
            if entry is None:
                return shoot(404, "No such user")
            return JSONResponse(entry.json())



import discord
from typing import *
from royalnet.commands import *


class SkipCommand(Command):
    name: str = "skip"

    aliases = ["s"]

    description: str = "Salta il file attualmente in riproduzione."

    async def run(self, args: CommandArgs, data: CommandData) -> None:
        if self.interface.name == "discord":
            message: discord.Message = data.message
            guild: discord.Guild = message.guild
            guild_id: Optional[int] = guild.id
        else:
            guild_id = None
        response: Dict[str, Any] = await self.interface.call_herald_event("discord", "discord_skip", guild_id=guild_id)
        await data.reply("⏩ File attuale saltato!")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['royalpack',
 'royalpack.commands',
 'royalpack.events',
 'royalpack.stars',
 'royalpack.tables',
 'royalpack.utils']

package_data = \
{'': ['*']}

install_requires = \
['riotwatcher>=2.7.1,<3.0.0',
 'royalnet[telegram,discord,alchemy_easy,bard,constellation,sentry,herald,coloredlogs]>=5.1.4,<6.0.0',
 'royalspells>=3.2,<4.0']

setup_kwargs = {
    'name': 'royalpack',
    'version': '5.1.6',
    'description': 'A Royalnet command pack for the Royal Games community',
    'long_description': '# `royalpack` [![PyPI](https://img.shields.io/pypi/v/royalpack.svg)](https://pypi.org/project/royalpack/)\n\nThe Royalnet Pack used in the Royal Games community!\n',
    'author': 'Stefano Pigozzi',
    'author_email': 'ste.pigozzi@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Steffo99/royalpack',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import typing
import urllib.parse
import asyncio
from royalnet.commands import *
from royalnet.utils import asyncify
from royalnet.audio import YtdlMp3


class Mp3Command(Command):
    name: str = "mp3"

    aliases = ["dlmusic"]

    description: str = "Scarica un video con youtube-dl e invialo in chat."

    syntax = "{ytdlstring}"

    ytdl_args = {
        "format": "bestaudio",
        "outtmpl": f"./downloads/%(title)s.%(ext)s"
    }

    seconds_before_deletion = 15 * 60

    async def run(self, args: CommandArgs, data: CommandData) -> None:
        url = args.joined()
        if url.startswith("http://") or url.startswith("https://"):
            vfiles: typing.List[YtdlMp3] = await asyncify(YtdlMp3.create_and_ready_from_url,
                                                          url,
                                                          **self.ytdl_args)
        else:
            vfiles = await asyncify(YtdlMp3.create_and_ready_from_url, f"ytsearch:{url}", **self.ytdl_args)
        for vfile in vfiles:
            await data.reply(f"⬇️ Il file richiesto può essere scaricato a:\n"
                             f"https://scaleway.steffo.eu/{urllib.parse.quote(vfile.mp3_filename.replace('./downloads/', './musicbot_cache/'))}\n"
                             f"Verrà eliminato tra {self.seconds_before_deletion} secondi.")
        await asyncio.sleep(self.seconds_before_deletion)
        for vfile in vfiles:
            vfile.delete()
            await data.reply(f"⏹ Il file {vfile.info.title} è scaduto ed è stato eliminato.")



from typing import *
from royalnet.commands import *


class CvCommand(Command):
    name: str = "cv"

    description: str = "Elenca le persone attualmente connesse alla chat vocale."

    syntax: str = "[a][o][n][d][h]"

    def __init__(self, interface: CommandInterface):
        super().__init__(interface)

    def _render_member(self,
                       member,
                       display_nick: bool,
                       display_discrim: bool):
        member: Dict[str, Any]
        status: str = ""
        if member["bot"]:
            status = "🤖"
        elif member["status"]["main"] == "online":
            status = "🔵"
        elif member["status"]["main"] == "idle":
            status = "⚫️"
        elif member["status"]["main"] == "dnd":
            status = "🔴"
        elif member["status"]["main"] == "offline":
            status = "⚪️"

        mvoice: Dict[str, Any] = member["voice"]
        voice: str = ""
        if mvoice is not None:
            if mvoice["server_mute"] or mvoice["server_deaf"]:
                voice = "🎵"
            elif mvoice["video"]:
                voice = "🖥"
            elif mvoice["afk"]:
                voice = "💤"
            elif mvoice["self_deaf"]:
                voice = "🔇"
            elif mvoice["self_mute"]:
                voice = "🔈"
            else:
                voice = "🔊"

        if display_nick and member["nick"] is not None:
            name = f"[i]{member['nick']}[/i]"
        elif display_discrim:
            name = f"{member['name']}#{member['discriminator']}"
        else:
            name = member['name']

        activity = ""
        for mact in member["activities"]:
            # Spotify
            if "type" not in mact:
                activity += f" | 🎧 {mact['details']} ({mact['state']})"
            # Playing
            elif mact["type"] == 0:
                activity += f" | 🎮 {mact['name']}"
                if "state" in mact and "details" in mact:
                    activity += f" ({mact['state']} | {mact['details']})"
                elif "state" in mact:
                    activity += f" ({mact['state']})"
                elif "details" in mact:
                    activity += f" ({mact['details']})"
            # Streaming
            elif mact["type"] == 1:
                activity += f" | 🎥 {mact['name']}"
            # Listening
            elif mact["type"] == 2:
                activity += f" | 🎧 {mact['name']}"
            # Watching
            elif mact["type"] == 3:
                activity += f" | 📺 {mact['name']}"
            # Custom Status
            elif mact["type"] == 4:
                activity += f" | ❓ {mact['state']}"
            else:
                raise ExternalError(f"Unknown Discord activity type: {mact['type']}")

        return f"{status}{voice} {name}{activity}\n"

    async def run(self, args: CommandArgs, data: CommandData) -> None:
        response: Dict[str, Any] = await self.interface.call_herald_event("discord", "discord_cv")

        flags = args.optional(0, default="")
        display_nicks = "n" in flags
        display_discrim = "d" in flags
        display_only_role = "a" not in flags
        display_only_online = "o" not in flags
        display_not_connected = "h" not in flags

        # Find all categories
        categories = []
        for member in response["guild"]["members"]:
            if member["voice"] is None:
                continue
            category = member["voice"]["channel"]["category"]
            if category not in categories:
                categories.append(category)
        categories.sort(key=lambda c: c["position"])

        # Find all channels, grouped by category id
        channels = {}
        for member in response["guild"]["members"]:
            if member["voice"] is None:
                continue
            category: Optional[int] = member["voice"]["channel"]["category"]["id"]
            if category not in channels:
                channels[category] = []
            channel = member["voice"]["channel"]
            if member["voice"]["channel"] not in channels[category]:
                channels[category].append(channel)
        for l in channels.values():
            l.sort(key=lambda c: c["position"])

        # Find all members, grouped by channel id
        members = {}
        not_connected_members = []
        for member in response["guild"]["members"]:
            if member["voice"] is None:
                not_connected_members.append(member)
                continue
            channel = member["voice"]["channel"]["id"]
            if channel not in members:
                members[channel] = []
            members[channel].append(member)
        for l in members.values():
            l.sort(key=lambda m: m["name"])

        # Construct the chat message
        message = f"ℹ️ Membri di [i]{response['guild']['name']}[/i]:\n\n"

        for category in categories:
            category: Dict[str, Any]
            if category['id'] is None:
                message += f"[b][Nessuna categoria][/b]\n"
            else:
                message += f"[b][{category['name']}][/b]\n"

            for channel in channels[category["id"]]:
                channel: Dict[str, Any]
                message += f"[b]#{channel['name']}[/b]\n"

                for member in members[channel["id"]]:
                    message += self._render_member(member, display_nicks, display_discrim)

                message += "\n"

        if display_not_connected and len(not_connected_members) >= 0:
            message += "[b][Non in chat vocale][/b]\n"
            for member in not_connected_members:

                draw = True

                for role in member["roles"]:
                    if role["id"] == self.interface.config["Cv"]["displayed_role_id"]:
                        break
                else:
                    if display_only_role:
                        draw = False

                if display_only_online and member["status"]["main"] == "offline":
                    draw = False

                if draw:
                    message += self._render_member(member, display_nicks, display_discrim)

        await data.reply(message)



import typing
import discord
import asyncio
import datetime
from royalnet.commands import *
from royalnet.utils import asyncify
from royalnet.audio import YtdlDiscord
from royalnet.audio.playmodes import Playlist
from royalnet.bots import DiscordBot


class ZawarudoCommand(Command):
    name: str = "zawarudo"

    aliases = ["theworld", "world"]

    description: str = "Ferma il tempo!"

    syntax = "[ [guild] ] [1-9]"

    @staticmethod
    async def _legacy_zawarudo_handler(bot: "DiscordBot", guild_name: typing.Optional[str], time: int):
        # Find the matching guild
        if guild_name:
            guilds: typing.List[discord.Guild] = bot.client.find_guild_by_name(guild_name)
        else:
            guilds = bot.client.guilds
        if len(guilds) == 0:
            raise CommandError("Server non trovato.")
        if len(guilds) > 1:
            raise CommandError("Il nome del server è ambiguo.")
        guild = list(bot.client.guilds)[0]
        # Ensure the guild has a PlayMode before adding the file to it
        if not bot.music_data.get(guild):
            raise CommandError("Il bot non è in nessun canale vocale.")
        # Create url
        ytdl_args = {
            "format": "bestaudio",
            "outtmpl": f"./downloads/{datetime.datetime.now().timestamp()}_%(title)s.%(ext)s"
        }
        # Start downloading
        zw_start: typing.List[YtdlDiscord] = await asyncify(YtdlDiscord.create_from_url,
                                                            "https://scaleway.steffo.eu/jojo/zawarudo_intro.mp3",
                                                            **ytdl_args)
        zw_end: typing.List[YtdlDiscord] = await asyncify(YtdlDiscord.create_from_url,
                                                          "https://scaleway.steffo.eu/jojo/zawarudo_outro.mp3",
                                                          **ytdl_args)
        old_playlist = bot.music_data[guild]
        bot.music_data[guild].playmode = Playlist()
        # Get voice client
        vc: discord.VoiceClient = bot.client.find_voice_client_by_guild(guild)
        channel: discord.VoiceChannel = vc.channel
        affected: typing.List[typing.Union[discord.User, discord.Member]] = channel.members
        await bot.add_to_music_data(zw_start, guild)
        for member in affected:
            if member.bot:
                continue
            await member.edit(mute=True)
        await asyncio.sleep(time)
        await bot.add_to_music_data(zw_end, guild)
        for member in affected:
            member: typing.Union[discord.User, discord.Member]
            if member.bot:
                continue
            await member.edit(mute=False)
        bot.music_data[guild] = old_playlist
        await bot.advance_music_data(guild)
        return {}

    _event_name = "_legacy_zawarudo"

    def __init__(self, interface: CommandInterface):
        super().__init__(interface)
        if interface.name == "discord":
            interface.register_herald_action(self._event_name, self._legacy_zawarudo_handler)

    async def run(self, args: CommandArgs, data: CommandData) -> None:
        guild_name, time = args.match(r"(?:\[(.+)])?\s*(.+)?")
        if time is None:
            time = 5
        else:
            time = int(time)
        if time < 1:
            raise InvalidInputError("The World can't stop time for less than a second.")
        if time > 10:
            raise InvalidInputError("The World can stop time only for 10 seconds.")
        await data.reply(f"🕒 ZA WARUDO! TOKI WO TOMARE!")
        await self.interface.call_herald_action("discord", self._event_name, {
                                                    "guild_name": guild_name,
                                                    "time": time
                                                })



from starlette.requests import Request
from starlette.responses import *
from royalnet.constellation import *
from royalnet.utils import *
from royalnet.backpack.tables import *


class ApiUserGetStar(PageStar):
    path = "/api/user/get/{uid_str}"

    async def page(self, request: Request) -> JSONResponse:
        uid_str = request.path_params.get("uid_str", "")
        try:
            uid = int(uid_str)
        except (ValueError, TypeError):
            return shoot(400, "Invalid uid")
        async with self.alchemy.session_acm() as session:
            user: User = await asyncify(session.query(self.alchemy.get(User)).get, uid)
        if user is None:
            return shoot(404, "No such user")
        return JSONResponse(user.json())



import typing
import discord
from royalnet.commands import *
from royalnet.audio.playmodes import Playlist, Pool, Layers
from royalnet.bots import DiscordBot


class PlaymodeCommand(Command):
    name: str = "playmode"

    aliases = ["pm", "mode"]

    description: str = "Cambia modalità di riproduzione per la chat vocale."

    syntax = "[ [guild] ] {mode}"

    @staticmethod
    async def _legacy_playmode_handler(bot: "DiscordBot", guild_name: typing.Optional[str], mode_name: str):
        """Handle a playmode Royalnet request. That is, change current PlayMode."""
        # Find the matching guild
        if guild_name:
            guilds: typing.List[discord.Guild] = bot.client.find_guild_by_name(guild_name)
        else:
            guilds = bot.client.guilds
        if len(guilds) == 0:
            raise CommandError("No guilds with the specified name found.")
        if len(guilds) > 1:
            raise CommandError("Multiple guilds with the specified name found.")
        guild = list(bot.client.guilds)[0]
        # Delete the previous PlayMode, if it exists
        if bot.music_data[guild] is not None:
            bot.music_data[guild].playmode.delete()
        # Create the new PlayMode
        if mode_name == "playlist":
            bot.music_data[guild].playmode = Playlist()
        elif mode_name == "pool":
            bot.music_data[guild].playmode = Pool()
        elif mode_name == "layers":
            bot.music_data[guild].playmode = Layers()
        else:
            raise CommandError("Unknown PlayMode specified.")
        return {}

    _event_name = "_legacy_playmode"

    def __init__(self, interface: CommandInterface):
        super().__init__(interface)
        if interface.name == "discord":
            interface.register_herald_action(self._event_name, self._legacy_playmode_handler)

    async def run(self, args: CommandArgs, data: CommandData) -> None:
        guild_name, mode_name = args.match(r"(?:\[(.+)])?\s*(\S+)\s*")
        await self.interface.call_herald_action("discord", self._event_name, {
                                                    "guild_name": guild_name,
                                                    "mode_name": mode_name
                                                })
        await data.reply(f"🔃 Impostata la modalità di riproduzione a: [c]{mode_name}[/c].")
